<?php
// $Id: nodereference_explorer.widget.inc,v 1.3 2009/07/17 15:38:28 gnindl Exp $
/**
 * @file
 * nodereferene_explorer.widget.inc
 * Widget hooks are placed here. This is the heart class of the module, as it builds on
 * top of the autocomplete widget. It add two buttons (browse and remove). The browse
 * button pops up an interactive dialog. All selected values are written to the autocomplete
 * widget.
 */

/**
 * Implementation of hook_widget_info().
 *
 * Adds the widget type explorer to the
 * nodereference field type
 * @return array of widget types
 */
function nodereference_explorer_widget_info() {
  return array(
    'nodereference_explorer' => array(
      'label' => t('Explorer'),
      'field types' => array('nodereference'),
    ),
  );
}

/**
 * Implementation of hook_widget().
 *
 * This method attaches a single form element to the form. It will be
 * built out and validated in the callback(s) listed in hook_elements.
 * We build it out in the callbacks rather than here in hook_widget so
 * it can be plugged into any module that can provide it with valid
 * $field information.
 *
 * Content module will set the weight, field name and delta values
 * for each form element. This is a change from earlier CCK versions
 * where the widget managed its own multiple values.
 *
 * If there are multiple values for this field, the content module will
 * call this function as many times as needed.
 *
 * @param $form
 *   the entire form array, $form['#node'] holds node information
 * @param $form_state
 *   the form_state, $form_state['values'][$field['field_name']]
 *   holds the field's form values.
 * @param $field
 *   the field array
 * @param $items
 *   array of default values for this field
 * @param $delta
 *   the order of this item in the array of subelements (0, 1, 2, etc)
 *
 * @return
 *   the form item for a single element for this field
 */
function nodereference_explorer_widget(&$form, &$form_state, $field, $items, $delta = 0) {
  switch ($field['widget']['type']) {
    case NODE_REFERENCE_EXPLORER_MODULE_NAME:
      //core widget (autocomplete)
      $element = array(
        '#type' => NODE_REFERENCE_EXPLORER_MODULE_NAME,
        '#default_value' => isset($items[$delta]) ? $items[$delta] : NULL,
        '#value_callback' => 'nodereference_explorer_widget_value',
      );
      break;
  }
  return $element;
}

/**
 * Proceses a single form element here.
 * @param $element
 *   form widget
 * @param $edit
 * @param $form_state
 *   the form_state, $form_state['values'][$field['field_name']] holds the field's form values.
 * @param $form
 *   the entire form array, $form['#node'] holds node information
 * @return $element
 *   widget including javascript
 */
function nodereference_explorer_widget_process($element, $edit, $form_state, $form) {

  $field = content_fields($element['#field_name'], $element['#type_name']);

  //add actions, e. g. browse and remove, to the core widget
  nodereference_explorer_widget_actions(&$element, $field);

  //add a preview if wished (by the content type admin/editor)
  if (!empty($field['widget']['content_preview']) && $field['widget']['content_preview'] != '--')
    nodereference_explorer_widget_preview(&$element, $field);

    //add popups & reference to each item
  if (module_exists('popups_reference') && module_exists('popups'))
    _nodereference_explorer_widget_add_popups_references($element, $field);

  return $element;
}

/**
 * Actions are attached to the core widget form element,
 * f. e. open content browser or remove selected value. The
 * necessary JS settings for the actions are added as well.
 * @param $element
 * 	parent widget, e. g. autocomplete
 * @param field
 *  CCK field
 */
function nodereference_explorer_widget_actions(&$element, $field) {

  //wrapper for actions bar
  $element['actions']['#prefix'] = '<div class="nodereference-explorer-actions">';
  $element['actions']['#suffix'] = '</div>';

  //Button ids
  $browse_button_id = $element['#id'] . '-button-browse';
  $remove_button_id = $element['#id'] . '-button-remove';

  //create buttons as form elements
  $element['actions']['browse'] = array(
    '#type' => 'nodereference_explorer_button',
    '#id' => $browse_button_id,
    '#attributes' => array('class' => 'nodereference-explorer-action-open-dialog nodereference-explorer-button'),
    '#value' => t('Browse'),
  );

  $element['actions']['remove'] = array(
    '#type' => 'nodereference_explorer_button',
    '#id' => $remove_button_id,
    '#value' => t('Remove'),
    '#attributes' => array('class' => 'nodereference-explorer-action-remove-value nodereference-explorer-button'),
  );

  //add button settings
  //TODO: When processing the "Add more item" requests, settings are passed
  //multiple times. The attributes, e. g. path, become arrays instead of
  //objects resulting in an JS s.url error. A manual type transformation is done in the explorer.js
  //TODO: provide the settings per field, not per field value
  //get parent widget id
  $widget_id = _nodereference_explorer_widget_generate_id($element);

  $setting[NODE_REFERENCE_EXPLORER_MODULE_NAME] = array(
    $browse_button_id => array(
      'path' => url(NODE_REFERENCE_EXPLORER_MENU_CALLBACK_DIALOG .'/'. $element['#type_name'] .'/'. $element['#field_name']),
      'widget' => $widget_id,
      'type' => $field['type'],
      'api' => nodereference_explorer_util_dialog_api($field),
      'modalframe' => url(NODE_REFERENCE_EXPLORER_MENU_CALLBACK_MODALFRAME .'/'. $element['#type_name'] .'/'. $element['#field_name']),
    ),
    $remove_button_id => array(
      'path' => url(NODE_REFERENCE_EXPLORER_MENU_CALLBACK_DIALOG .'/'. $element['#type_name'] .'/'. $element['#field_name']),
      'widget' => $widget_id,
      'type' => $field['type'],
    ),
  );
  drupal_add_js($setting, 'setting');

  //preload the theme due to better performance and working in IE 8
  $theme = nodereference_explorer_util_parse_settings($field, 'theme'); //theme mode
  nodereference_explorer_util_theme_css($theme); //load necessary css files
}
/**
 * Default preview on selected view items. Actually
 * an item of a referenced view display is shown
 * @param $element
 *   form element
 * @param $field
 *   CCK field
 */
function nodereference_explorer_widget_preview(&$element, $field) {
  $element['preview'] = array(
    '#type' => 'markup',
    '#weight' => $field['widget']['content_preview_position'] == 'below' ?  1 : -1,
  );

  $widget_id = _nodereference_explorer_widget_generate_id($element);
  $preview_id = $element['#id'] .'-preview';

  //add an empty placeholder for the preview
  $element['preview']['#value'] = '<div id="'. $preview_id .'" class="nodereference-explorer-preview"></div>';

  //TODO: provide the settings per field, not per field value
  $setting[NODE_REFERENCE_EXPLORER_MODULE_NAME] = array(
    $preview_id => array(
      'path' => url(NODE_REFERENCE_EXPLORER_MENU_CALLBACK_PREVIEW .'/'. $element['#type_name'] .'/'. $element['#field_name']),
      'widget' => $widget_id,
    ),
  );
  drupal_add_js($setting, 'setting'); //js settings
  nodereference_explorer_add_js('explorer.preview'); //preview behavior js
}

/**
 *
 * Value for a nodereference widget element.
 * Now it just calls the autocomplate value. Any form
 * related manipulation should be done here if necessary.
 * @param $element
 *   autocomplete widget
 * @param $edit
 * @return node
 *   name and id, eg. name [nid: xx]
 */
function nodereference_explorer_widget_value($element,  $edit = FALSE) {
  $result = nodereference_autocomplete_value($element, $edit);
  return $result;
}
/**
 * Computes a widget id for each item.
 * @param $element
 *   from element
 * @return
 *   generic node widget id
 */
function _nodereference_explorer_widget_generate_id($element) {
  $widget_id = $element['#id']; //generic actions, dialog can be used by third party modules, e. g. link
  if (isset($element['nid'])) //native actions
    $widget_id .= '-nid-nid';
  return $widget_id;
}

/**
 * Adds popups references to the widget if enabled
 * Copied and adapted from popups_reference_alter_item()
 */
function _nodereference_explorer_widget_add_popups_references(&$element, $field) {
  if (variable_get('popups_reference_show_add_link_'. $element['#field_name'], TRUE)) {
    $wrapper_id = 'popups-reference-' . _popups_reference_counter();
    $links = _popups_reference_links($field, 'nodereference_explorer', $wrapper_id, 'nodereference_autocomplete');
    if ($links) {
      // Put the nodereference widget and links in an wrapper.
      // Makes it easy to find for Ahah targeting, and popups_reference behavior selecting.
      $element['#prefix'] .= '<div class="'. $wrapper_id .'">';
      $element['#suffix'] .= '<div>Add New: ' . implode(', ', $links) .'</div></div>';
    }
  }
}