<?php
// $Id: nodereference_explorer.widget.integrate.inc,v 1.1 2009/07/17 15:38:28 gnindl Exp $

/**
 * @filenodereference_explorer.widget.integrate.inc
 * Supplies the nodereference_explorer widget for reuse of third party modules,
 * e. g. link.
 * Inspired by popups_reference. A similar approach is used here.
 */

/**
 * Field settings for integration, e. g. show widget and other dialog settings
 * @param $form
 * @param $form_state
 */
function nodereference_explorer_widget_integrate_field_settings(&$form, $form_state) {
  $field_name = $form['#field']['field_name'];
  $form['field']['show_nodereference_explorer_widget'] = array(
    '#type' => 'checkbox',
    '#default_value' => variable_get('nodereference_explorer_show_widget_'. $field_name, TRUE),
    '#title' => t('Enable Nodereference Explorer'),
    '#description' => t('Adds widget actions to the CCK field.'),
  );

  $form['field']['nodereference_explorer_settings'] = array(
    '#title' => t('Nodereference Explorer settings'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $widget = array(
    'type' => NODE_REFERENCE_EXPLORER_MODULE_NAME,
    'label' => $form['#field']['widget']['label'],
  );

  $settings = variable_get('nodereference_explorer_settings_'. $field_name, NULL);
  if (isset($settings) && !empty($settings))
    $widget = array_merge($widget, $settings);

  $form['field']['nodereference_explorer_settings'][] = nodereference_explorer_widget_settings_form($widget);
  //remove autocomplete settings
  unset($form['field']['nodereference_explorer_settings']['0']['autocomplete_match']);
  unset($form['field']['nodereference_explorer_settings']['0']['size']);

  $form['field']['nodereference_explorer_settings'][] = nodereference_field_settings('form', $settings);

  $form['#submit'][] = '_nodereference_explorer_widget_integrate_manage_fields_submit';

}

/**
 * Attach browse and remove buttons to fields.
 * @param $form
 * @param $form_state
 */
function nodereference_explorer_widget_integrate_node_form(&$form, $form_state) {
  $fields = content_fields();
  foreach ($form as $key => $item) {
    if (is_array($item)) {
      $type = $item['#type'];
      if ($type == 'fieldset') { // Loop through all the subitems.
        foreach ($form[$key] as $subkey => $subitem) {
          nodereference_explorer_widget_integrate_alter_item($form[$key], $subkey, $subitem, $fields);
        }
      }
      else {
        nodereference_explorer_widget_integrate_alter_item($form, $key, $item, $fields);
      }
    }
  }
}

/**
 * Run on every element in the basic node form.
 * Wrap the enabled nodereference fields, and add the popup links.
 *
 * @param $form - the form (or fieldgroup).
 * @param $key - form element name.
 * @param $item - the form element array.
 * @param $fields - all fields info.
 */
function nodereference_explorer_widget_integrate_alter_item(&$form, $key, $item, $fields) {
  $field_name = strstr($key, 'field_'); // Check if $key starts with 'field_';
  if (isset($fields[$field_name]) &&
    $fields[$field_name]['type'] == 'link' &&
    variable_get('nodereference_explorer_show_widget_'. $field_name, FALSE)) {
      foreach ($item as $id => $value) {
        if (is_int($id)) {
          $element['#id'] = form_clean_id('edit-' . $value['#field_name'] . '-' . $value['#delta']); //'edit-field-fds-0-url';
          $element['#field_name'] = $fields[$field_name]['field_name'];
          $element['#type_name'] = $fields[$field_name]['type_name'];
          //add buttons
          nodereference_explorer_widget_actions(&$element, $fields[$field_name]);
          //TODO: support preview for external fields
         // $fields[$field_name]['widget']['content_preview_position'] = 'below';
         // nodereference_explorer_widget_preview(&$element, $fields[$field_name]);

         // $form[$key][$id]['#suffix'] =  $element['preview']['#value'];
          //render buttons
          $form[$key][$id]['#suffix'] = $element['actions']['#prefix'];
          $form[$key][$id]['#suffix'] .= theme('nodereference_explorer_button', $element['actions']['browse']);
          $form[$key][$id]['#suffix'] .= theme('nodereference_explorer_button', $element['actions']['remove']);
          //fake core widget field
          //a JS change event is attached to this hidden field
          //and values to be set anywhere you like
          $form[$key][$id]['#suffix'] .= '<input type="hidden" id="' . $element['#id'] .'"/>';
          $form[$key][$id]['#suffix'] .= $element['actions']['#suffix'];
        }
      }
  }
}

/**
 * Submit added to the the third party module settings form.
 * Set a variable for each external field.
 * @param $form
 * @param $form_state
 */
function _nodereference_explorer_widget_integrate_manage_fields_submit($form, &$form_state) {
  $field_name = $form['#field']['field_name'];
  variable_set('nodereference_explorer_show_widget_'. $field_name, $form_state['values']['show_nodereference_explorer_widget']);
  //_nodereference_explorer_widget_parse_settings()
 $settings = array();
 foreach ($form_state['values'] as $key => $value) {
   if (strstr($key, 'dialog_') || strstr($key, 'content_') || strstr($key, 'theme_') ||
       strstr($key, 'advanced_view') || strstr($key, 'referenceable_types') || strstr($key, 'advanced_view_args'))
         $settings[$key] = $value;
 }
 variable_set('nodereference_explorer_settings_'. $field_name, $settings);
}