// $Id: explorer.js,v 1.3 2009/07/17 15:38:28 gnindl Exp $

/**
 * @file explorer.js
 * The nodereference explorer javascript file.
 * It's the core javascript file which is always included. Besides a
 * behavior for the core widget it is responsible for attaching js
 * and css files inline, i. e. on a JSON request.
 */
//define name space
Drupal.nodereference_explorer = {};

//cache added javascript sources and stylesheets
Drupal.nodereference_explorer.addedJS = [];
Drupal.nodereference_explorer.addedCSS = [];

/**
 * Behavior of core widget
 * @param
 *   DOM context
 */
Drupal.behaviors.NodereferenceExplorer = function(context) {
   //add css class to surrounding wrapper, needed for theming
  $('.nodereference-explorer .form-autocomplete', context).parent().addClass('nodereference-explorer-wrapper');
}

/**
 * Get the nodereference_explorer settings. A fix is included here that transforms
 * the wrong settings arrays back to object.
 * @param id of the action
 * @return
 *   array of settings
 */
Drupal.nodereference_explorer.getSettings = function(id) {
  var settings = {};
  for (var prop in Drupal.settings.nodereference_explorer[id]) {
	//FIX: Settings are populated twice during AHAH. This leads to confunsion in JS
    if (Drupal.settings.nodereference_explorer[$id][prop] instanceof Array) 
      settings[prop] = Drupal.settings.nodereference_explorer[$id][prop][0]; //array contains equals values, take the first
    else {
      settings[prop] = Drupal.settings.nodereference_explorer[$id][prop];
    }
  }
  return settings;
}

/**
 * Add additional CSS to the page (see popups.js).
 */
Drupal.nodereference_explorer.addCSS = function(css) {
  for (var type in css) {
    for (var file in css[type]) {
      var link = css[type][file];
      // Does the page already contain this stylesheet?
      if (!$('link[href='+ $(link).attr('href') + ']').length) {
        $('head').append(link);
         Drupal.nodereference_explorer.addedCSS.push(link); // Keep a list, so we can remove them later.
      }
    }
  }
};

/**
 * Add additional Javascript to the page (see popups.js).
 */
Drupal.nodereference_explorer.addJS = function(js) {
  // Parse the json info about the new context.
  var scripts = [];
  var inlines = [];
  for (var type in js) {
    if (type != 'setting') {
      for (var file in js[type]) {
    	  //console.log(file);
        if (type == 'inline') {
          inlines.push($(js[type][file]).text());
        }
        else {
          scripts.push($(js[type][file]).attr('src'));
        }
      }
    }
  }

  // Add new JS settings to the page, needed for #ahah properties to work.
  $.extend(Drupal.settings, js.setting);

  for (var i in scripts) {
    var src = scripts[i];
    if (!$('script[src='+ src + ']').length && !Drupal.nodereference_explorer.addedJS[src]) {
      // Get the script from the server and execute it.
      $.ajax({
        type: 'GET',
        url: src,
        dataType: 'script',
        async : false,
        success: function(script) {
          eval(script);
        }
      });
      // Mark the js as added to the underlying page.
      Drupal.nodereference_explorer.addedJS[src] = true;
    }
  }
  return inlines;
};

/**
 * Execute the jit loaded inline scripts  (see popups.js).
 * Q: Do we want to re-excute the ones already in the page?
 *
 * @source
 *   stolen straight from Popups API function Popups.addInlineJS
 * @param inlines
 *   Array of inline scripts.
 */
Drupal.nodereference_explorer.addInlineJS = function(inlines) {
  // Load the inlines into the page.
  for (var n in inlines) {
    // If the script is not already in the page, execute it.
    if (!$('script:not([src]):contains(' + inlines[n] + ')').length) {
      eval(inlines[n]);
    }
  }
};